from dataclasses import dataclass
from typing import Callable, Iterable

from cml.ports.source_adapters import Adapter
from cml.ports.scikit_adapter import MachineLearningModel
from cml.domain.reconstruction import Reconstructor
from cml.domain.data_source import DataSource
from cml.domain.construction import Constructor
from cml.shared.settings import (
    PreprocessingSettings,
    BlockProcessingSettings,
    DeconstructionSettings,
    ConstructionSettings
)


@dataclass
class PreprocessRequest:
    source_adapter: Adapter
    pre_settings: PreprocessingSettings
    block_settings: BlockProcessingSettings
    density_estimator: MachineLearningModel
    relative_extrema: Callable


@dataclass
class KnowledgeSearchRequest:
    constructor: Constructor
    reconstructor: Reconstructor
    data_source: DataSource
    deconstruction_settings: DeconstructionSettings


@dataclass
class CreateConstructorRequest:
    settings: ConstructionSettings
    ml_models: Iterable
